package io.redvox.api900;

import net.jpountz.lz4.LZ4Compressor;
import net.jpountz.lz4.LZ4Factory;
import net.jpountz.lz4.LZ4FastDecompressor;
import net.jpountz.lz4.LZ4SafeDecompressor;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Arrays;

public class Lz4 {
    private static LZ4Factory lz4Factory = null;
    private static LZ4Compressor lz4HighCompressor = null;
    private static LZ4Compressor lz4FastCompressor = null;
    private static LZ4FastDecompressor lz4FastDecompressor = null;
    private static LZ4SafeDecompressor lz4SafeDecompressor = null;


    public static synchronized LZ4Factory getLz4Factory() {
        if(lz4Factory == null) {
            lz4Factory = LZ4Factory.fastestInstance();
        }
        return lz4Factory;
    }

    public static synchronized LZ4Compressor getLz4HighCompressor() {
        if(lz4HighCompressor == null) {
            lz4HighCompressor = getLz4Factory().highCompressor();
        }
        return lz4HighCompressor;
    }

    public static synchronized LZ4Compressor getLz4FastCompressor() {
        if(lz4FastCompressor == null) {
            lz4FastCompressor = getLz4Factory().fastCompressor();
        }
        return lz4FastCompressor;
    }

    public static synchronized LZ4FastDecompressor getLz4FastDecompressor() {
        if(lz4FastDecompressor == null) {
            lz4FastDecompressor = getLz4Factory().fastDecompressor();
        }
        return lz4FastDecompressor;
    }

    public static synchronized LZ4SafeDecompressor getLz4SafeDecompressor() {
        if(lz4SafeDecompressor == null) {
            lz4SafeDecompressor = getLz4Factory().safeDecompressor();
        }
        return lz4SafeDecompressor;
    }

    public static byte[] compress(final byte[] bytes) {
        return compress(getLz4HighCompressor(), bytes);
    }

    public static byte[] compress(final LZ4Compressor compressor, final byte[] bytes) {
        final byte[] uncompressedSize = encodeUncompressedSize(bytes.length);
        final byte[] compressedData = compressor.compress(bytes);
        final byte[] payload = new byte[uncompressedSize.length + compressedData.length];
        System.arraycopy(uncompressedSize, 0, payload, 0, uncompressedSize.length);
        System.arraycopy(compressedData, 0, payload, uncompressedSize.length, compressedData.length);
        return payload;
    }

    public static void compressFile(final Path from, final Path to) throws IOException {
        final byte[] originalBytes = Files.readAllBytes(from);
        final byte[] compressedBytes = compress(originalBytes);
        Files.write(to, compressedBytes);
    }

    public static void decompressFileFast(final Path from, final Path to) throws IOException {
        final byte[] compressedBytes = Files.readAllBytes(from);
        final byte[] decompressed = decompressFast(compressedBytes);
        Files.write(to, decompressed);
    }

    public static void decompressFileSafe(final Path from, final Path to) throws IOException {
        final byte[] compressedBytes = Files.readAllBytes(from);
        final byte[] decompressed = decompressSafe(compressedBytes);
        Files.write(to, decompressed);
    }

    public static byte[] decompressFast(final byte[] bytes) {
        return decompressFast(getLz4FastDecompressor(), bytes);
    }

    public static byte[] decompressFast(final LZ4FastDecompressor decompressor, final byte[] bytes) {
        final int uncompressedSize = decodeUncompressedSize(bytes);
        return decompressFast(decompressor, bytes, uncompressedSize);
    }

    public static byte[] decompressFast(final LZ4FastDecompressor decompressor, final byte[] bytes, final int uncompressedSize) {
        final byte[] compressedData = Arrays.copyOfRange(bytes, 4, bytes.length);
        final byte[] result = new byte[uncompressedSize];
        decompressor.decompress(compressedData, result);
        return result;
    }

    public static byte[] decompressSafe(final byte[] bytes) {
        return decompressSafe(getLz4SafeDecompressor(), bytes);
    }

    public static byte[] decompressSafe(final LZ4SafeDecompressor decompressor, final byte[] bytes) {
        final int uncompressedSize = decodeUncompressedSize(bytes);
        return decompressSafe(decompressor, bytes, uncompressedSize);
    }

    public static byte[] decompressSafe(final LZ4SafeDecompressor decompressor, final byte[] bytes, final int uncompressedSize) {
        final byte[] compressedData = Arrays.copyOfRange(bytes, 4, bytes.length);
        return decompressor.decompress(compressedData, uncompressedSize);
    }


    public static int decodeUncompressedSize(final byte[] bytes) {
        int ret = 0;
        ret |= (int)bytes[0] & 0xFF;
        ret <<= 8;
        ret |= (int)bytes[1] & 0xFF;
        ret <<= 8;
        ret |= (int)bytes[2] & 0xFF;
        ret <<= 8;
        ret |= (int)bytes[3] & 0xFF;
        return ret;
    }

    public static byte[] encodeUncompressedSize(final int size) {
        final byte[] result = new byte[4];
        result[0] = (byte)(size >> 24);
        result[1] = (byte)(size >> 16);
        result[2] = (byte)(size >> 8);
        result[3] = (byte) size;
        return result;
    }
}
