package io.redvox.api900;

import io.redvox.apis.Api900;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.function.Function;

public class PayloadUtil {
    private static <T, R> List<R> extract(final List<T> list, final int idx, final int step, final Function<T, R> transform) {
        final List<R> extracted = new ArrayList<>();
        for (int i = idx; i < list.size(); i += step) {
            extracted.add(transform.apply(list.get(i)));
        }
        return extracted;
    }

    private enum JointPayloadCase {
        BYTE_PAYLOAD,
        UINT32_PAYLOAD,
        UINT64_PAYLOAD,
        INT32_PAYLOAD,
        INT64_PAYLOAD,
        FLOAT32_PAYLOAD,
        FLOAT64_PAYLOAD,
        PAYLOAD_NOT_SET;
    }

    private static JointPayloadCase from(final Api900.EvenlySampledChannel evenlySampledChannel) {
        return JointPayloadCase.valueOf(evenlySampledChannel.getPayloadCase().name());
    }

    private static JointPayloadCase from(final Api900.UnevenlySampledChannel unevenlySampledChannel) {
        return JointPayloadCase.valueOf(unevenlySampledChannel.getPayloadCase().name());
    }

    private static <R> List<R> extractPayload(final Api900.EvenlySampledChannel evenlySampledChannel,
                                              final Api900.UnevenlySampledChannel unevenlySampledChannel,
                                              final int channelIdx,
                                              final int step,
                                              final Function<Byte, R> byteTransform,
                                              final Function<Integer, R> integerTransform,
                                              final Function<Long, R> longTransform,
                                              final Function<Float, R> floatTransform,
                                              final Function<Double, R> doubleTransform) {

        final JointPayloadCase payloadCase = evenlySampledChannel != null ? from(evenlySampledChannel)
                : from(unevenlySampledChannel);

        switch (payloadCase) {
            case BYTE_PAYLOAD:
                final List<R> byteVals = new ArrayList<>();
                final byte[] bytes = evenlySampledChannel != null ? evenlySampledChannel.getBytePayload().getPayload().toByteArray() :
                        unevenlySampledChannel.getBytePayload().getPayload().toByteArray();
                for (int i = channelIdx; i < bytes.length; i += step) {
                    byteVals.add(byteTransform.apply(bytes[i]));
                }
                return byteVals;
            case UINT32_PAYLOAD:
                return PayloadUtil.extract(
                        evenlySampledChannel != null ? evenlySampledChannel.getUint32Payload().getPayloadList()
                                : unevenlySampledChannel.getUint32Payload().getPayloadList(),
                        channelIdx, step, integerTransform);
            case UINT64_PAYLOAD:
                return PayloadUtil.extract(
                        evenlySampledChannel != null ? evenlySampledChannel.getUint64Payload().getPayloadList()
                                : unevenlySampledChannel.getUint64Payload().getPayloadList(),
                        channelIdx, step, longTransform);
            case INT32_PAYLOAD:
                return PayloadUtil.extract(
                        evenlySampledChannel != null ? evenlySampledChannel.getInt32Payload().getPayloadList()
                                : unevenlySampledChannel.getInt32Payload().getPayloadList(),
                        channelIdx, step, integerTransform);
            case INT64_PAYLOAD:
                return PayloadUtil.extract(
                        evenlySampledChannel != null ? evenlySampledChannel.getInt64Payload().getPayloadList()
                                : unevenlySampledChannel.getInt64Payload().getPayloadList(),
                        channelIdx, step, longTransform);
            case FLOAT32_PAYLOAD:
                return PayloadUtil.extract(
                        evenlySampledChannel != null ? evenlySampledChannel.getFloat32Payload().getPayloadList()
                                : unevenlySampledChannel.getFloat32Payload().getPayloadList(),
                        channelIdx, step, floatTransform);
            case FLOAT64_PAYLOAD:
                return PayloadUtil.extract(
                        evenlySampledChannel != null ? evenlySampledChannel.getFloat64Payload().getPayloadList()
                                : unevenlySampledChannel.getFloat64Payload().getPayloadList(),
                        channelIdx, step, doubleTransform);
            case PAYLOAD_NOT_SET:
                return Collections.emptyList();
            default:
                return Collections.emptyList();
        }
    }

    private static List<Double> extractFloatingPayload(final Api900.EvenlySampledChannel evenlySampledChannel,
                                                       final Api900.UnevenlySampledChannel unevenlySampledChannel,
                                                       final int channelIdx,
                                                       final int step) {
        return extractPayload(evenlySampledChannel,
                unevenlySampledChannel,
                channelIdx,
                step,
                Byte::doubleValue,
                Integer::doubleValue,
                Long::doubleValue,
                Float::doubleValue,
                d -> d);
    }

    private static List<Double> extractFloatingPayload(final Api900.EvenlySampledChannel evenlySampledChannel,
                                                       final int channelIdx,
                                                       final int step) {
        return extractFloatingPayload(evenlySampledChannel, null, channelIdx, step);
    }

    public static List<Double> extractFloatingPayload(final Api900.EvenlySampledChannel evenlySampledChannel) {
        return extractFloatingPayload(evenlySampledChannel, 0, 1);
    }

    public static List<Double> extractFloatingPayload(final Api900.UnevenlySampledChannel unevenlySampledChannel,
                                                      final int channelIdx,
                                                      final int step) {
        return extractFloatingPayload(null, unevenlySampledChannel, channelIdx, step);
    }

    public static List<Double> extractFloatingPayload(final Api900.UnevenlySampledChannel unevenlySampledChannel) {
        return extractFloatingPayload(unevenlySampledChannel, 0, 1);
    }

    private static List<Long> extractIntegralPayload(final Api900.EvenlySampledChannel evenlySampledChannel,
                                                     final Api900.UnevenlySampledChannel unevenlySampledChannel,
                                                     final int channelIdx,
                                                     final int step) {
        return extractPayload(evenlySampledChannel,
                unevenlySampledChannel,
                channelIdx,
                step,
                Byte::longValue,
                Integer::longValue,
                l -> l,
                Float::longValue,
                Double::longValue);
    }

    private static List<Long> extractIntegralPayload(final Api900.EvenlySampledChannel evenlySampledChannel,
                                                     final int channelIdx,
                                                     final int step) {
        return extractIntegralPayload(evenlySampledChannel, null, channelIdx, step);
    }

    public static List<Long> extractIntegralPayload(final Api900.EvenlySampledChannel evenlySampledChannel) {
        return extractIntegralPayload(evenlySampledChannel, 0, 1);
    }

    private static List<Long> extractIntegralPayload(final Api900.UnevenlySampledChannel unevenlySampledChannel,
                                                     final int channelIdx,
                                                     final int step) {
        return extractIntegralPayload(null, unevenlySampledChannel, channelIdx, step);
    }

    public static List<Long> extractIntegralPayload(final Api900.UnevenlySampledChannel unevenlySampledChannel) {
        return extractIntegralPayload(unevenlySampledChannel, 0, 1);
    }

    private static List<Byte> extractBytePayload(final byte[] bytes,
                                                 final int channelIdx,
                                                 final int step) {
        final List<Byte> byteList = new ArrayList<>();
        for (int i = channelIdx; i < bytes.length; i++) {
            byteList.add(bytes[i]);
        }
        return byteList;
    }


    public static List<Byte> extractBytePayload(final Api900.EvenlySampledChannel evenlySampledChannel,
                                                final int channelIdx,
                                                final int step) {
        return extractBytePayload(evenlySampledChannel.getBytePayload().getPayload().toByteArray(),
                channelIdx,
                step);
    }

    public static List<Byte> extractBytePayload(final Api900.UnevenlySampledChannel unevenlySampledChannel,
                                                final int channelIdx,
                                                final int step) {
        return extractBytePayload(unevenlySampledChannel.getBytePayload().getPayload().toByteArray(),
                channelIdx,
                step);
    }
}
